@@ -33,6 +33,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 namespace KWin
 {
 
+extern int screen_number; // main.cpp
+
 EglOnXBackend::EglOnXBackend()
     : OpenGLBackend()
     , m_overlayWindow(new OverlayWindow())
@@ -74,9 +76,9 @@ void EglOnXBackend::init()
     }
 
     initEGL();
-    if (!hasGLExtension(QStringLiteral("EGL_KHR_image")) &&
-        (!hasGLExtension(QStringLiteral("EGL_KHR_image_base")) ||
-         !hasGLExtension(QStringLiteral("EGL_KHR_image_pixmap")))) {
+    if (!hasGLExtension(QByteArrayLiteral("EGL_KHR_image")) &&
+        (!hasGLExtension(QByteArrayLiteral("EGL_KHR_image_base")) ||
+         !hasGLExtension(QByteArrayLiteral("EGL_KHR_image_pixmap")))) {
         setFailed(QStringLiteral("Required support for binding pixmaps to EGLImages not found, disabling compositing"));
         return;
     }
@@ -89,13 +91,13 @@ void EglOnXBackend::init()
         options->setGlPreferBufferSwap('e'); // for unknown drivers - should not happen
     glPlatform->printResults();
     initGL(EglPlatformInterface);
-    if (!hasGLExtension(QStringLiteral("GL_OES_EGL_image"))) {
+    if (!hasGLExtension(QByteArrayLiteral("GL_OES_EGL_image"))) {
         setFailed(QStringLiteral("Required extension GL_OES_EGL_image not found, disabling compositing"));
         return;
     }
 
     // check for EGL_NV_post_sub_buffer and whether it can be used on the surface
-    if (eglPostSubBufferNV) {
+    if (hasGLExtension(QByteArrayLiteral("EGL_NV_post_sub_buffer"))) {
         if (eglQuerySurface(dpy, surface, EGL_POST_SUB_BUFFER_SUPPORTED_NV, &surfaceHasSubPost) == EGL_FALSE) {
             EGLint error = eglGetError();
             if (error != EGL_SUCCESS && error != EGL_BAD_ATTRIBUTE) {
@@ -109,7 +111,7 @@ void EglOnXBackend::init()
 
     setSupportsBufferAge(false);
 
-    if (hasGLExtension("EGL_EXT_buffer_age")) {
+    if (hasGLExtension(QByteArrayLiteral("EGL_EXT_buffer_age"))) {
         const QByteArray useBufferAge = qgetenv("KWIN_USE_BUFFER_AGE");
 
         if (useBufferAge != "0")
@@ -158,7 +160,34 @@ void EglOnXBackend::init()
 
 bool EglOnXBackend::initRenderingContext()
 {
-    dpy = eglGetDisplay(display());
+    // Get the list of client extensions
+    const QByteArray clientExtensionString = eglQueryString(EGL_NO_DISPLAY, EGL_EXTENSIONS);
+    if (clientExtensionString.isEmpty()) {
+        // If eglQueryString() returned NULL, the implementation doesn't support
+        // EGL_EXT_client_extensions. Expect an EGL_BAD_DISPLAY error.
+        (void) eglGetError();
+    }
+
+    const QList<QByteArray> clientExtensions = clientExtensionString.split(' ');
+
+    // Use eglGetPlatformDisplayEXT() to get the display pointer
+    // if the implementation supports it.
+    const bool havePlatformBase = clientExtensions.contains("EGL_EXT_platform_base");
+    if (havePlatformBase) {
+        // Make sure that the X11 platform is supported
+        if (!clientExtensions.contains("EGL_EXT_platform_x11"))
+            return false;
+
+        const int attribs[] = {
+            EGL_PLATFORM_X11_SCREEN_EXT, screen_number,
+            EGL_NONE
+        };
+
+        dpy = eglGetPlatformDisplayEXT(EGL_PLATFORM_X11_EXT, display(), attribs);
+    } else {
+        dpy = eglGetDisplay(display());
+    }
+
     if (dpy == EGL_NO_DISPLAY)
         return false;
 
@@ -184,7 +213,16 @@ bool EglOnXBackend::initRenderingContext()
         overlayWindow()->setup(None);
     }
 
-    surface = eglCreateWindowSurface(dpy, config, overlayWindow()->window(), 0);
+    if (havePlatformBase) {
+        // Note: Window is 64 bits on a 64-bit architecture whereas xcb_window_t is
+        //       always 32 bits. eglCreatePlatformWindowSurfaceEXT() expects the
+        //       native_window parameter to be pointer to a Window, so this variable
+        //       cannot be an xcb_window_t.
+        const Window window = overlayWindow()->window();
+        surface = eglCreatePlatformWindowSurfaceEXT(dpy, config, (void *) &window, nullptr);
+    } else {
+        surface = eglCreateWindowSurface(dpy, config, overlayWindow()->window(), nullptr);
+    }
 
 #ifdef KWIN_HAVE_OPENGLES
     const EGLint context_attribs[] = {
@@ -459,17 +497,11 @@ OpenGLBackend *EglTexture::backend()
     return m_backend;
 }
 
-void EglTexture::findTarget()
+bool EglTexture::loadTexture(xcb_pixmap_t pix, const QSize &size, xcb_visualid_t visual)
 {
-    if (m_target != GL_TEXTURE_2D) {
-        m_target = GL_TEXTURE_2D;
-    }
-}
+    Q_UNUSED(visual)
 
-bool EglTexture::loadTexture(const Pixmap &pix, const QSize &size, int depth)
-{
-    Q_UNUSED(depth)
-    if (pix == None)
+    if (pix == XCB_NONE)
         return false;
 
     glGenTextures(1, &m_texture);
